from flask import Flask, jsonify, render_template
import logging
import requests
import yaml
from dotenv import load_dotenv
import os
import json
from datetime import datetime, timedelta
from flask_socketio import SocketIO

app = Flask(__name__)
app.config['SECRET_KEY'] = 'secret!'
socketio = SocketIO(app)

# Load environment variables from .env file
load_dotenv()

# Set up logging
logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)

TOKEN_FILE = 'access_token.json'

def save_token_to_json(token, expiry_time, file_path=TOKEN_FILE):
    """Save the access token to a JSON file."""
    token_data = {
        'access_token': token,
        'expiry_time': expiry_time.isoformat()  # Save the expiry time as an ISO formatted string
    }
    with open(file_path, 'w') as file:
        json.dump(token_data, file)
    logger.info(f"Access token saved to {file_path}")

def load_token_from_json(file_path=TOKEN_FILE):
    """Load the access token from a JSON file."""
    if os.path.exists(file_path):
        with open(file_path, 'r') as file:
            token_data = json.load(file)
            token_data['expiry_time'] = datetime.fromisoformat(token_data['expiry_time'])  # Convert expiry time back to datetime
            return token_data
    return None

def get_access_token(config):
    existing_token_data = load_token_from_json()
    if existing_token_data and existing_token_data['expiry_time'] > datetime.now():
        logger.info("Using existing access token")
        return existing_token_data['access_token']
    
    logger.info("Requesting a new access token")
    url = f"{os.getenv('PROD')}{config['api_config']['access_token']['url']}"
    headers = {
        'content-type': config['api_config']['access_token']['headers']['content-type']
    }
    body = {
        'grant_type': os.getenv('GRANT_TYPE'),
        'client_id': os.getenv('CLIENT_ID'),  # Added client_id
        'appkey': os.getenv('PROD_APPKEY'),
        'appsecret': os.getenv('PROD_APPSECRET')
    }

    # Debug logging to check environment variables
    logger.debug(f"Environment variables: GRANT_TYPE={os.getenv('GRANT_TYPE')}, CLIENT_ID={os.getenv('CLIENT_ID')}, PROD_APPKEY={os.getenv('PROD_APPKEY')}, PROD_APPSECRET={os.getenv('PROD_APPSECRET')}")

    # Construct the URL-encoded request body
    data = '&'.join([f'{key}={value}' for key, value in body.items()])

    # Make the network request
    logger.info(f"Requesting access token from URL: {url}")
    logger.info(f"Headers: {headers}")
    logger.info(f"Body: {data}")

    response = requests.post(url, headers=headers, data=data)

    # Log the response details
    logger.info(f"Response status code: {response.status_code}")
    logger.info(f"Response text: {response.text}")

    if response.status_code == 200:
        response_data = response.json()
        token = response_data.get('access_token')
        expires_in = response_data.get('expires_in', 3600)  # Default to 1 hour if not provided
        expiry_time = datetime.now() + timedelta(seconds=expires_in)
        save_token_to_json(token, expiry_time)  # Save the token and its expiry time
        return token
    else:
        raise Exception(f"Failed to get access token: {response.status_code}, {response.text}")

def fetch_balance(config, access_token):
    url = f"{os.getenv('PROD')}{config['api_config']['stock_balance']['url']}"
    headers = {
        'content-type': config['api_config']['stock_balance']['headers']['content-type'],
        'authorization': f"Bearer {access_token}",
        'appkey': os.getenv('PROD_APPKEY'),
        'appsecret': os.getenv('PROD_APPSECRET'),
        'tr_id': config['api_config']['stock_balance']['headers']['tr_id']
    }
    params = {
        'CANO': os.getenv('CANO_REAL'),
        'ACNT_PRDT_CD': os.getenv('CANO_REAL_T'),
        'AFHR_FLPR_YN': config['api_config']['stock_balance']['query_params']['AFHR_FLPR_YN'],
        'OFL_YN': config['api_config']['stock_balance']['query_params']['OFL_YN'],
        'INQR_DVSN': config['api_config']['stock_balance']['query_params']['INQR_DVSN'],
        'UNPR_DVSN': config['api_config']['stock_balance']['query_params']['UNPR_DVSN'],
        'FUND_STTL_ICLD_YN': config['api_config']['stock_balance']['query_params']['FUND_STTL_ICLD_YN'],
        'FNCG_AMT_AUTO_RDPT_YN': config['api_config']['stock_balance']['query_params']['FNCG_AMT_AUTO_RDPT_YN'],
        'PRCS_DVSN': config['api_config']['stock_balance']['query_params']['PRCS_DVSN'],
        'COST_ICLD_YN': config['api_config']['stock_balance']['query_params']['COST_ICLD_YN'],
        'CTX_AREA_FK100': config['api_config']['stock_balance']['query_params']['CTX_AREA_FK100'],
        'CTX_AREA_NK100': config['api_config']['stock_balance']['query_params']['CTX_AREA_NK100']
    }

    logger.info(f"Requesting stock balance from URL: {url}")
    logger.info(f"Headers: {headers}")
    logger.info(f"Params: {params}")

    response = requests.get(url, headers=headers, params=params)

    logger.info(f"Response status code: {response.status_code}")
    logger.info(f"Response text: {response.text}")

    if response.status_code == 200:
        return response.json()
    else:
        raise Exception(f"Failed to fetch balance: {response.status_code}, {response.text}")

@app.route('/')
def index():
    return render_template('index.html')

@app.route('/balance-data')
def balance_data():
    try:
        # Read settings from 'config.yaml' file
        config_path = './config.yaml'
        with open(config_path, 'r') as file:
            config = yaml.safe_load(file)

        # Obtain access token
        access_token = get_access_token(config)
        logger.info(f"Access token obtained: {access_token}")

        # Fetch stock balance
        balance_data = fetch_balance(config, access_token)
        return jsonify(balance_data)
    except Exception as e:
        logger.error(f"Error: {e}")
        return jsonify({'error': str(e)}), 500

@socketio.on('connect')
def handle_connect():
    print('Client connected')
    # Emit initial portfolio data or other relevant data if needed
    # This can be fetched from a database or other storage if needed

@socketio.on('disconnect')
def handle_disconnect():
    print('Client disconnected')

if __name__ == "__main__":
    socketio.run(app, debug=True)
