
import pandas as pd
from sklearn.ensemble import RandomForestClassifier
from sklearn.model_selection import train_test_split
from sklearn.preprocessing import StandardScaler
import numpy as np
import joblib
import os
import logging

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)

MODEL_FILE = 'optimized_model.pkl'

def generate_optimized_signals(data, sma_params, ema_params, rsi_params):
    signals = data.copy()
    signals['signal'] = 0  # 기본값
    
    # Calculate SMA
    signals['sma'] = signals['price'].rolling(window=sma_params['window'], min_periods=1).mean()
    
    # Calculate EMA
    signals['ema'] = signals['price'].ewm(span=ema_params['span'], adjust=False).mean()
    
    # Calculate RSI
    delta = signals['price'].diff()
    gain = (delta.where(delta > 0, 0)).rolling(window=rsi_params['window']).mean()
    loss = (-delta.where(delta < 0, 0)).rolling(window=rsi_params['window']).mean()
    rs = gain / loss
    signals['rsi'] = 100 - (100 / (1 + rs))
    
    signals['signal'][(signals['sma'] > signals['ema']) & (signals['rsi'] < rsi_params['threshold'])] = 1
    signals['signal'][(signals['sma'] < signals['ema']) & (signals['rsi'] > rsi_params['threshold'])] = -1

    return signals

def train_optimized_model(data, sma_params, ema_params, rsi_params):
    data = data.dropna()
    features = data[['price', 'sma', 'ema', 'rsi']]
    labels = data['signal']
    
    X_train, X_test, y_train, y_test = train_test_split(features, labels, test_size=0.2, random_state=42)
    
    scaler = StandardScaler()
    X_train = scaler.fit_transform(X_train)
    X_test = scaler.transform(X_test)

    model = RandomForestClassifier(n_estimators=100, random_state=42)
    model.fit(X_train, y_train)

    accuracy = model.score(X_test, y_test)
    logger.info(f"Model accuracy: {accuracy}")

    joblib.dump(model, MODEL_FILE)
    logger.info(f"Optimized model saved to {MODEL_FILE}")

    return model

def load_optimized_model():
    if os.path.exists(MODEL_FILE):
        model = joblib.load(MODEL_FILE)
        logger.info(f"Loaded optimized model from {MODEL_FILE}")
        return model
    else:
        raise FileNotFoundError(f"Model file not found: {MODEL_FILE}")
