
import tkinter as tk
from tkinter import messagebox
import asyncio
import aiohttp
import yaml
import logging
from app import get_access_token  # Import the get_access_token function from app.py

# Configure logging
logging.basicConfig(level=logging.INFO, format='%(asctime)s - %(levelname)s - %(message)s')

# Load configuration
def load_config():
    with open('config.yaml', 'r', encoding='utf-8') as file:
        config = yaml.safe_load(file)
    return config

# Get API details
def get_api_details(config):
    api_details = config['trading_bot']['api']
    return api_details

# Fetch account information
async def fetch_account_info(api_details):
    base_url = api_details['base_url']
    access_token = get_access_token()  # Get the access token

    headers = {
        'Authorization': f'Bearer {access_token}',
        'appKey': api_details['app_key'],
        'appSecret': api_details['app_secret']
    }

    async with aiohttp.ClientSession() as session:
        async with session.get(f"{base_url}/uapi/domestic-stock/v1/trading/inquire-balance", headers=headers) as response:
            if response.status == 200:
                return await response.json()
            else:
                logging.error(f"계좌 정보 가져오기 오류: {response.status}")
                return None

# Update account information
async def update_account_info(api_details, label):
    while True:
        data = await fetch_account_info(api_details)
        if data:
            info = (
                f"총 손익: {data['output'][0]['evlu_pfls_amt']}\n"
                f"총 매입: {data['output'][0]['pchs_amt_smtl']}\n"
                # 필요에 따라 추가 라인 포함
            )
            label.config(text=info)
        await asyncio.sleep(5)  # Update every 5 seconds

# Fetch real-time data
async def fetch_real_time_data(api_details):
    base_url = api_details['base_url']
    access_token = get_access_token()  # Get the access token

    headers = {
        'Authorization': f'Bearer {access_token}',
        'appKey': api_details['app_key'],
        'appSecret': api_details['app_secret']
    }

    async with aiohttp.ClientSession() as session:
        async with session.get(f"{base_url}/uapi/domestic-stock/v1/quotations/inquire-price", headers=headers) as response:
            if response.status == 200:
                return await response.json()
            else:
                logging.error(f"데이터 가져오기 오류: {response.status}")
                return None

# Analyze data
async def analyze_data(data):
    logging.info("데이터 분석 중...")
    return {"signal": "buy", "symbol": "AAPL", "quantity": 10}

# Full sell
async def full_sell(api_details):
    base_url = api_details['base_url']
    access_token = get_access_token()  # Get the access token

    headers = {
        'Authorization': f'Bearer {access_token}',
        'appKey': api_details['app_key'],
        'appSecret': api_details['app_secret']
    }

    async with aiohttp.ClientSession() as session:
        async with session.get(f"{base_url}/uapi/domestic-stock/v1/trading/inquire-balance", headers=headers) as response:
            if response.status != 200:
                logging.error(f"보유 자산 가져오기 오류: {response.status}")
                return

            holdings = await response.json()

        tasks = []
        for holding in holdings['output']:
            symbol = holding['pdno']
            quantity = holding['hldg_qty']

            order_payload = {
                'CANO': api_details['account_number'],
                'ACNT_PRDT_CD': '01',
                'PDNO': symbol,
                'ORD_DVSN': '00',
                'ORD_QTY': quantity,
                'ORD_UNPR': '0'
            }

            tasks.append(place_order(session, base_url, headers, order_payload))

        await asyncio.gather(*tasks)

async def place_order(session, base_url, headers, order_payload):
    async with session.post(f"{base_url}/uapi/domestic-stock/v1/trading/order-cash", json=order_payload, headers=headers) as response:
        if response.status == 200:
            logging.info("주문 성공")
        else:
            logging.error(f"주문 실패: {response.status}")

# Main function to start the bot
def main():
    config = load_config()
    api_details = get_api_details(config)
    root = tk.Tk()
    root.title("Stock Trading Bot")
    label = tk.Label(root, text="계좌 정보를 불러오는 중...")
    label.pack()
    asyncio.run(update_account_info(api_details, label))
    root.mainloop()

if __name__ == "__main__":
    main()
